@@ -6,6 +6,7 @@ from OFS.CopySupport import CopyError
 from OFS.event import ObjectWillBeMovedEvent
 from OFS.interfaces import IObjectWillBeMovedEvent
 from plone import api
+from plone.api.content import _parse_object_provides_query
 from plone.api.content import NEW_LINKINTEGRITY
 from plone.api.tests.base import INTEGRATION_TESTING
 from plone.app.layout.navigation.interfaces import INavigationRoot
@@ -112,6 +113,28 @@ class TestPloneApiContent(unittest.TestCase):
             id='image',
         )
 
+    def verify_intids(self):
+        """Test that the intids are in order"""
+        from zope.component import getUtility
+        from zope.intid.interfaces import IIntIds
+
+        intids = getUtility(IIntIds)
+        broken_keys = [
+            key for key in intids.ids
+            if not self.portal.unrestrictedTraverse(key.path, None)
+        ]
+        obsolete_paths = [key.path for key in broken_keys]
+        self.assertListEqual(obsolete_paths, [])
+
+        # Objects used as keys with a hash can behave strangely.
+        # I have seen this go wrong in a production site.
+        weird_keys = [
+            key for key in intids.ids
+            if key not in intids.ids
+        ]
+        weird_paths = [key.path for key in weird_keys]
+        self.assertListEqual(weird_paths, [])
+
     def test_create_constraints(self):
         """Test the constraints when creating content."""
         from plone.api.exc import InvalidParameterError
@@ -241,6 +264,7 @@ class TestPloneApiContent(unittest.TestCase):
                 type='Dexterity Item',
                 id='test-item',
             )
+        self.verify_intids()
 
     def test_create_content(self):
         """Test create content"""
@@ -288,6 +312,7 @@ class TestPloneApiContent(unittest.TestCase):
                 type='Document',
                 id='test-document',
             )
+        self.verify_intids()
 
     def test_create_with_safe_id(self):
         """Test the content creating with safe_id mode."""
@@ -551,6 +576,7 @@ class TestPloneApiContent(unittest.TestCase):
             container['events']['about']
             and container['events']['about'] == about
         )
+        self.verify_intids()
 
     def test_move_no_move_if_target_is_source_parent(self):
         """Test that trying to move an object to its parent is a noop"""
@@ -1049,6 +1075,74 @@ class TestPloneApiContent(unittest.TestCase):
         )
         self.assertEqual(len(brains), 1)
 
+    def test_find_interface_dict__include_not_query(self):
+        """Check if not query in object_provides is functional.
+        """
+
+        brains_all = api.content.find(
+            object_provides={'query': IContentish.__identifier__},
+        )
+
+        alsoProvides(self.portal.events, INavigationRoot)
+        self.portal.events.reindexObject(idxs=['object_provides'])
+
+        brains = api.content.find(
+            object_provides={
+                'query': IContentish.__identifier__,
+                'not': INavigationRoot.__identifier__,
+            },
+        )
+
+        self.assertEqual(len(brains_all) - len(brains), 1)
+
+    def test_find_interface_dict__all_options(self):
+        """ Check for all options in a object_provides query are correctly
+        transformed.
+        """
+        parser = _parse_object_provides_query
+
+        self.assertDictEqual(
+            parser({'query': IContentish}),
+            {'query': [IContentish.__identifier__], 'operator': 'or'},
+        )
+
+        self.assertDictEqual(
+            parser(
+                {
+                    'query': [IContentish, INavigationRoot.__identifier__],
+                    'operator': 'and',
+                },
+            ),
+            {
+                'query': [IContentish.__identifier__, INavigationRoot.__identifier__],
+                'operator': 'and',
+            },
+        )
+
+        self.assertDictEqual(
+            parser({'not': IContentish}),
+            {'not': [IContentish.__identifier__]},
+        )
+
+        self.assertDictEqual(
+            parser({'not': [IContentish, INavigationRoot.__identifier__]}),
+            {'not': [IContentish.__identifier__, INavigationRoot.__identifier__]},
+        )
+
+        self.assertDictEqual(
+            parser({'not': IContentish}),
+            {'not': [IContentish.__identifier__]},
+        )
+
+        self.assertDictEqual(
+            parser({'query': IContentish, 'operator': 'and', 'not': INavigationRoot}),
+            {
+                'query': [IContentish.__identifier__],
+                'operator': 'and',
+                'not': [INavigationRoot.__identifier__],
+            },
+        )
+
     def test_find_dict(self):
         # Pass arguments using dict
         path = '/'.join(self.portal.about.getPhysicalPath())
